//***************************** POCKET.CPP *********************************

// Compiler directives
#include										<dos.h>
#include                          <conio.h>
#include                          <stdio.h>
#include                         <stdlib.h>
#include                         <string.h>
#include                           <time.h>
#include                          <ctype.h>

// Others...
#define BUFFER_SIZE                     130

// data_fields[] constants
#define NUMBER_SAMPLES                    0
#define MILLISECONDS                      1
#define SECONDS                           2
#define MINUTES                           3
#define HOURS                             4
#define DAYS                              5
#define FILE_NAME                         6
#define PORT_ADDRESS                      7
#define REFERENCE_VOLTAGE                 8
#define TOTAL_SECONDS                     9


// Prototypes
float convert_data(unsigned char);
int read_config(char*, struct config_strings*);
unsigned char get_sample(void);
int check_sampler(void);
void do_sampling(int);
void save_data(void);

// Set up some global variables
//
struct config_strings
{
	 char * name ;
	 char * data ;
};

config_strings data_fields[] =
{
	 {"number_samples"	,NULL},// All fields _MUST_ be lower case !!!
	 {"milliseconds"  	,NULL},
	 {"seconds"   			,NULL},
	 {"minutes"    		,NULL},
	 {"hours"       		,NULL},
	 {"days"        		,NULL},
	 {"file_name"     	,NULL},
	 {"port_address"  	,NULL},
	 {"reference_voltage",NULL},
	 {NULL            	,NULL} // Array terminator. REQUIRED !!!
};

unsigned char data[10000];

//---------------------read_config-----------------------------------------
//
int read_config(char *filename, struct config_strings *config_info )
{
	char   buffer[BUFFER_SIZE];
	char   *work_pointer;
	char   *config_name;
	char   *config_data;
	struct config_strings *cfg;
	FILE   *config_file;

	printf("\nUsing config file: %s\n",filename);

	config_file=fopen(filename,"rt");

	if(config_file==NULL)
	{
		return EXIT_FAILURE;// Couldn't open cfg file
	}

	while((fgets(buffer, BUFFER_SIZE, config_file )!=NULL))
	{
		//	clip off optional comment tail indicated by a semi-colon
		if((work_pointer=strchr(buffer,';'))!=NULL)
		{
			*work_pointer='\0';
		}
		else
		{
			work_pointer=buffer+strlen(buffer);
		}

		//	clip off trailing and leading white space
		work_pointer--;

		while(isspace(*work_pointer)&&work_pointer>=buffer)
		{
			*work_pointer-- ='\0';
		}

		work_pointer=buffer;

		while(isspace(*work_pointer)) work_pointer++;

		if(strlen(work_pointer)==0) continue;

		config_name=strtok(work_pointer," =");

		if(config_name!=NULL)
		{
			//	Condition the name (lower case required),
			//	and strip leading white and a 'late' = from data part.
			strlwr(config_name);
			config_data=strtok(NULL,"");

			while(isspace(*config_data)) config_data++;

			if(*config_data=='=') config_data++;

			while(isspace(*config_data)) config_data++;

			// look for matching 'name'
			cfg=config_info;

			while(NULL!=cfg->name&&0!=strcmp(cfg->name,config_name))
			{
				cfg++;
			}
			//	duplicate the data if the name is found.

			if(cfg->name!=NULL)
			{
				cfg->data=strdup(config_data);// strdup is not ANSI
														// memory leaks if cfg->data
														//	is malloc'ed already
				if(cfg->data==NULL)
				{
					fclose(config_file);
					return EXIT_FAILURE;
				}
			}   // undetected error on failure should not be a problem
		}		 // as configuration reading should be done early.
	}	       // but test and handle it anyway ...

	fclose(config_file);
	return 0;
}

//---------------------- do_sampling ---------------------------------------
//
void do_sampling(int scale)
{
	int i;
	time_t t;
	long total_seconds;

	total_seconds=	(atoi(data_fields[DAYS].data)   * 86400)+
						(atoi(data_fields[HOURS].data)  * 3600 )+
						(atoi(data_fields[MINUTES].data)* 60   )+
						 atoi(data_fields[SECONDS].data);

	ltoa(total_seconds,	data_fields[TOTAL_SECONDS].data,10);
	data_fields[TOTAL_SECONDS].name=NULL;

	time(&t);
		printf("\nSampling started on %s",ctime(&t));

	if(total_seconds<1)
	{
		printf("%d samples to be taken.\n",atoi(data_fields[NUMBER_SAMPLES].data));
		printf("Taking one sample every %d milliseconds\n",atoi(data_fields[MILLISECONDS].data));
		printf("Sampling now. Please wait...\n");

		for(i=0;i<atoi(data_fields[NUMBER_SAMPLES].data);i++)
		{
			data[i]=get_sample();
			delay(atoi(data_fields[MILLISECONDS].data));// in milliseconds
		}
	}
	else
	{
		printf("%d samples to be taken.\n",atoi(data_fields[NUMBER_SAMPLES].data));
		printf("Taking one sample every ");

		for(i=DAYS;i>MILLISECONDS;i--)
		{
			if (atoi(data_fields[i].data)) printf("%d %s,", atoi(data_fields[i].data), data_fields[i].name);
		}
		printf("\nPress Ctrl-C to abort.\n\nTaking sample number ");

		for(i=0;i<atoi(data_fields[NUMBER_SAMPLES].data);i++)
		{
			t=time(NULL);
			data[i]=get_sample();

			gotoxy(22,wherey());
			printf("%d: %2.3fV ",i+1,(convert_data(data[i])*scale)/2);//+1 for humans...

			while(time(NULL)-t<total_seconds)
			{
				// Insert data/sample averaging code here
			}
		}
		printf("\n\nSampling stopped on %s",ctime(&t));
		printf("%d samples taken.\n",i);
	}
}

//------------------------ check_sampler -----------------------------------
//
int check_sampler(void)
{
	int i, count=0, port=0;

	sscanf(data_fields[PORT_ADDRESS].data,"%x",&port);

	for(i=0;i<255;i++)
	{
		if((inp(port+1)&8)) count++;
	}

	return(count);
}

//------------------------get_scale ----------------------------------------
//
int get_scale(void)
{
	int port=0,in_port,out_port,scale;

	sscanf(data_fields[PORT_ADDRESS].data,"%x",&port);
	in_port =port+1;
	out_port=port+2;

	outp(out_port,253);              //OUT (Port + 2), 253

	scale=((inp(in_port)&128)?10:1);

	outp(out_port,2);                //OUT (Port + 2), 2
	outp(out_port,3);                //OUT (Port + 2), 3
	outp(out_port,6);                //OUT (Port + 2), 6

	return(scale); //Scale = 10^((INP(Port + 1) AND 128) / 128)
}

//------------------------ get_sample --------------------------------------
//
unsigned char get_sample(void)
{
	int lo, hi, port=0, in_port,out_port;

	sscanf(data_fields[PORT_ADDRESS].data,"%x",&port);
	in_port =port+1;
	out_port=port+2;

	outp(out_port,0x2);
	lo=inp(in_port);
	outp(out_port,0x3);
	hi=inp(in_port);
	outp(out_port,0x6);
	return(((lo>>4)|(hi&240))^0x88);
}


//------------------------------ convert_data -------------------------------
//
float convert_data(unsigned char data)
{
	return(((float)(atof(data_fields[REFERENCE_VOLTAGE].data)*2)/256)*data);
}


//------------------------------- save_data ---------------------------------
//
void save_data(int error_flag, int scale)
{
	FILE *output_file;
	char *output_file_name = data_fields[FILE_NAME].data;
	char timestring[10], datestring[10];
	int i;
	time_t t;

	time(&t);

	_strtime(timestring);
	_strdate(datestring);

	output_file = fopen(output_file_name, "wt");
	fprintf(output_file,"           Data file from POCKET.EXE\n");
	fprintf(output_file,"      Driver program for the Pocket Sampler.\n");
	fprintf(output_file,"       Electronics Australia, August 1996.\n");
	fprintf(output_file,"     File created at %s on %s\n",timestring, datestring);
	fprintf(output_file,"\n");
	fprintf(output_file,"\n");
	fprintf(output_file,"        Total number of samples =     %d\n",atoi(data_fields[NUMBER_SAMPLES].data));
	fprintf(output_file,"   Single sample time (seconds) =     %d\n",atoi(data_fields[TOTAL_SECONDS].data));
	fprintf(output_file,"Total sampling period (seconds) =     %d\n",atoi(data_fields[NUMBER_SAMPLES].data) * atoi(data_fields[TOTAL_SECONDS].data));
	fprintf(output_file,"          Voltage scale (volts) =     %d\n",scale*2);
	fprintf(output_file,"\n");

	if(error_flag==255)
	{
		fprintf(output_file,"\n*** ERROR: sampler didn't respond on port %s -- data invalid ***\n", data_fields[PORT_ADDRESS].data);
	}

	for (i=0;i<atoi(data_fields[NUMBER_SAMPLES].data);i++)
	{
	  fprintf(output_file, "%3.3f\n", (convert_data(data[i])*scale)/2);
	}

	fclose(output_file);
	printf("Data saved to %s.\n",output_file_name);
}

//----------------------------- generate_config -----------------------------
//
void generate_config(void)
{
	FILE *output_file;
	time_t t;

	printf("Generating new config file: POCKET.CFG\n");

	output_file = fopen("pocket.cfg","wt");

	fprintf(output_file,"; Default config file for POCKET.EXE\n");

	time(&t);
	fprintf(output_file,"; Generated on %s",ctime(&t));

	fprintf(output_file,"; Run POCKET.EXE /g to generate new config file\n\n");

	fprintf(output_file,"; --------------------------------------------------\n");

	fprintf(output_file,"; Total number of samples to be taken\n");
	fprintf(output_file,"number_samples    = 10\n\n");

	fprintf(output_file,"; Time delay between each sample\n");
	fprintf(output_file,"seconds           =  1\n");
	fprintf(output_file,"minutes           =  0\n");
	fprintf(output_file,"hours             =  0\n");
	fprintf(output_file,"days              =  0\n\n");

	fprintf(output_file,"; Time delay between each sample (in milliseconds)\n");
	fprintf(output_file,"; Note that if previous time values are\n");
	fprintf(output_file,"; non-zero, this setting will be ignored.\n");
	fprintf(output_file,"milliseconds      =  0\n\n");

	fprintf(output_file,"; Address of parallel port used by sampler\n");
	fprintf(output_file,"; (in hexadecimal)\n");
	fprintf(output_file,"port_address      =  378\n\n");

	fprintf(output_file,"; Sampler's internal reference voltage\n");
	fprintf(output_file,"; (depends on hardware, if in doubt, leave at 1)\n");
	fprintf(output_file,"reference_voltage =  1\n\n");

	fprintf(output_file,"; Default filename to save data.\n");
	fprintf(output_file,"; If a filename is specified on command line,\n");
	fprintf(output_file,"; this setting will be ignored\n");
	fprintf(output_file,"file_name         =  DEFAULT.PKT\n\n");

	fclose(output_file);
}

//=================================== main ==================================
int main(int argc, char *argv[])
{
	FILE   *input_file=NULL;
	char* filename=NULL,*config=NULL;
	int i=0, scale=10;

	printf("\n----------------------------------------\n");
	printf("Pocket Sampler textmode version 1.0\n");
	printf("  (c) 1998 Electronics Australia\n");

	while(i++<=argc)
	{
		if(argv[i][0]=='/')
		{

			switch(argv[i][1])
			{
				case 'f':
				case 'F': filename=argv[i+1]; break;

				case 'c':
				case 'C': config=argv[i+1]; break;

				case 'g':
				case 'G': generate_config(); exit(EXIT_FAILURE); break;

				case '?':
				default : printf("Usage: POCKET.EXE [[/f <output filename>] [/c <config filename>]] | [/g]\n\n");
							 printf("Example:POCKET.EXE /c CUSTOM.CFG /f DATAFILE.TXT\n\n");
							 printf("Using the /g switch will cause POCKET.EXE to re-generate the default\nconfig file POCKET.CFG. ");
							 printf("If run without any switches, POCKET.EXE will ");
							 printf("try to\nuse the configuration file POCKET.CFG in the current directory.\nIf this file is not present, ");
							 printf("a default POCKET.CFG will be created.\n");
							 printf("Command line switches will override any config file settings.");
							 exit(EXIT_FAILURE); break;
			}
		}
	}

	if (config==NULL)
	{
		if ((input_file = fopen("POCKET.CFG","rt"))!=NULL)
		{
			fclose(input_file);
			read_config("POCKET.CFG",data_fields);
		}
		else
		{
			printf("No config file specified!\n");
			generate_config();
			exit(EXIT_FAILURE);
		}
	}
	else
	{
		read_config(config,data_fields);
	}

	if(filename != NULL)
	{
		data_fields[FILE_NAME].data = filename;
	}


	if(check_sampler())
	{
		printf("\n*** ERROR: sampler isn't responding on port %s.\n", data_fields[PORT_ADDRESS].data);
		printf("*** Sampling anyway, error message will be logged in data file.\n");
		i=255;//Error flag for save routine.

	}
	scale=get_scale();
	printf("Scale: %d",scale*2);

	do_sampling(scale);
	save_data(i,scale);
	return(i);
}

//============================== END OF PROGRAM =============================